@@ -10,12 +10,19 @@
 
 #include "system.h"
 
+#if defined(__LCLINT__)
+/*@-declundef -exportheader @*/
+extern long long int strtoll(const char *nptr, /*@null@*/ char **endptr,
+		int base)
+	/*@modifies *endptr@*/;
+/*@=declundef =exportheader @*/
+#endif
+
 #ifdef HAVE_FLOAT_H
 #include <float.h>
 #endif
 #include <math.h>
 
-#include "findme.h"
 #include "poptint.h"
 
 #ifdef	MYDEBUG
@@ -56,9 +63,7 @@ void poptSetExecPath(poptContext con, const char * path, int allowAbsolute)
     con->execPath = _free(con->execPath);
     con->execPath = xstrdup(path);
     con->execAbsolute = allowAbsolute;
-/*@-nullstate@*/ /* LCL: con->execPath not NULL */
     return;
-/*@=nullstate@*/
 }
 
 static void invokeCallbacksPRE(poptContext con, const struct poptOption * opt)
@@ -67,24 +72,20 @@ static void invokeCallbacksPRE(poptContext con, const struct poptOption * opt)
 {
     if (opt != NULL)
     for (; opt->longName || opt->shortName || opt->arg; opt++) {
-	if (opt->arg == NULL) continue;		/* XXX program error. */
-	if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_INCLUDE_TABLE) {
-	    void * arg = opt->arg;
-/*@-branchstate@*/
-	    /* XXX sick hack to preserve pretense of ABI. */
-	    if (arg == poptHelpOptions) arg = poptHelpOptionsI18N;
-/*@=branchstate@*/
-	    /* Recurse on included sub-tables. */
-	    invokeCallbacksPRE(con, arg);
-	} else if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_CALLBACK &&
-		   (opt->argInfo & POPT_CBFLAG_PRE))
-	{   /*@-castfcnptr@*/
-	    poptCallbackType cb = (poptCallbackType)opt->arg;
-	    /*@=castfcnptr@*/
-	    /* Perform callback. */
-	    /*@-noeffectuncon @*/
-	    cb(con, POPT_CALLBACK_REASON_PRE, NULL, NULL, opt->descrip);
-	    /*@=noeffectuncon @*/
+	poptArg arg = { .ptr = opt->arg };
+	if (arg.ptr)
+	switch (poptArgType(opt)) {
+	case POPT_ARG_INCLUDE_TABLE:	/* Recurse on included sub-tables. */
+	    poptSubstituteHelpI18N(arg.opt);	/* XXX side effects */
+	    invokeCallbacksPRE(con, arg.opt);
+	    /*@switchbreak@*/ break;
+	case POPT_ARG_CALLBACK:		/* Perform callback. */
+	    if (!CBF_ISSET(opt, PRE))
+		/*@switchbreak@*/ break;
+/*@-noeffectuncon @*/	/* XXX no known way to annotate (*vector) calls. */
+	    arg.cb(con, POPT_CALLBACK_REASON_PRE, NULL, NULL, opt->descrip);
+/*@=noeffectuncon @*/
+	    /*@switchbreak@*/ break;
 	}
     }
 }
@@ -95,78 +96,72 @@ static void invokeCallbacksPOST(poptContext con, const struct poptOption * opt)
 {
     if (opt != NULL)
     for (; opt->longName || opt->shortName || opt->arg; opt++) {
-	if (opt->arg == NULL) continue;		/* XXX program error. */
-	if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_INCLUDE_TABLE) {
-	    void * arg = opt->arg;
-/*@-branchstate@*/
-	    /* XXX sick hack to preserve pretense of ABI. */
-	    if (arg == poptHelpOptions) arg = poptHelpOptionsI18N;
-/*@=branchstate@*/
-	    /* Recurse on included sub-tables. */
-	    invokeCallbacksPOST(con, arg);
-	} else if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_CALLBACK &&
-		   (opt->argInfo & POPT_CBFLAG_POST))
-	{   /*@-castfcnptr@*/
-	    poptCallbackType cb = (poptCallbackType)opt->arg;
-	    /*@=castfcnptr@*/
-	    /* Perform callback. */
-	    /*@-noeffectuncon @*/
-	    cb(con, POPT_CALLBACK_REASON_POST, NULL, NULL, opt->descrip);
-	    /*@=noeffectuncon @*/
+	poptArg arg = { .ptr = opt->arg };
+	if (arg.ptr)
+	switch (poptArgType(opt)) {
+	case POPT_ARG_INCLUDE_TABLE:	/* Recurse on included sub-tables. */
+	    poptSubstituteHelpI18N(arg.opt);	/* XXX side effects */
+	    invokeCallbacksPOST(con, arg.opt);
+	    /*@switchbreak@*/ break;
+	case POPT_ARG_CALLBACK:		/* Perform callback. */
+	    if (!CBF_ISSET(opt, POST))
+		/*@switchbreak@*/ break;
+/*@-noeffectuncon @*/	/* XXX no known way to annotate (*vector) calls. */
+	    arg.cb(con, POPT_CALLBACK_REASON_POST, NULL, NULL, opt->descrip);
+/*@=noeffectuncon @*/
+	    /*@switchbreak@*/ break;
 	}
     }
 }
 
 static void invokeCallbacksOPTION(poptContext con,
-				  const struct poptOption * opt,
-				  const struct poptOption * myOpt,
-				  /*@null@*/ const void * myData, int shorty)
+				const struct poptOption * opt,
+				const struct poptOption * myOpt,
+				/*@null@*/ const void * myData, int shorty)
 	/*@globals internalState@*/
 	/*@modifies internalState@*/
 {
     const struct poptOption * cbopt = NULL;
+    poptArg cbarg = { .ptr = NULL };
 
     if (opt != NULL)
     for (; opt->longName || opt->shortName || opt->arg; opt++) {
-	if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_INCLUDE_TABLE) {
-	    void * arg = opt->arg;
-/*@-branchstate@*/
-	    /* XXX sick hack to preserve pretense of ABI. */
-	    if (arg == poptHelpOptions) arg = poptHelpOptionsI18N;
-/*@=branchstate@*/
-	    /* Recurse on included sub-tables. */
-	    if (opt->arg != NULL)	/* XXX program error */
+	poptArg arg = { .ptr = opt->arg };
+	switch (poptArgType(opt)) {
+	case POPT_ARG_INCLUDE_TABLE:	/* Recurse on included sub-tables. */
+	    poptSubstituteHelpI18N(arg.opt);	/* XXX side effects */
+	    if (opt->arg != NULL)
 		invokeCallbacksOPTION(con, opt->arg, myOpt, myData, shorty);
-	} else if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_CALLBACK &&
-		  !(opt->argInfo & POPT_CBFLAG_SKIPOPTION)) {
-	    /* Save callback info. */
+	    /*@switchbreak@*/ break;
+	case POPT_ARG_CALLBACK:		/* Save callback info. */
+	    if (CBF_ISSET(opt, SKIPOPTION))
+		/*@switchbreak@*/ break;
 	    cbopt = opt;
-	} else if (cbopt != NULL &&
-		   ((myOpt->shortName && opt->shortName && shorty &&
-			myOpt->shortName == opt->shortName) ||
-		    (myOpt->longName != NULL && opt->longName != NULL &&
+	    cbarg.ptr = opt->arg;
+	    /*@switchbreak@*/ break;
+	default:		/* Perform callback on matching option. */
+	    if (cbopt == NULL || cbarg.cb == NULL)
+		/*@switchbreak@*/ break;
+	    if ((myOpt->shortName && opt->shortName && shorty &&
+			myOpt->shortName == opt->shortName)
+	     || (myOpt->longName != NULL && opt->longName != NULL &&
 			!strcmp(myOpt->longName, opt->longName)))
-		   )
-	{   /*@-castfcnptr@*/
-	    poptCallbackType cb = (poptCallbackType)cbopt->arg;
-	    /*@=castfcnptr@*/
-	    const void * cbData = (cbopt->descrip ? cbopt->descrip : myData);
-	    /* Perform callback. */
-	    if (cb != NULL) {	/* XXX program error */
-		/*@-noeffectuncon @*/
-		cb(con, POPT_CALLBACK_REASON_OPTION, myOpt,
-			con->os->nextArg, cbData);
-		/*@=noeffectuncon @*/
+	    {	const void *cbData = (cbopt->descrip ? cbopt->descrip : myData);
+/*@-noeffectuncon @*/	/* XXX no known way to annotate (*vector) calls. */
+		cbarg.cb(con, POPT_CALLBACK_REASON_OPTION,
+			myOpt, con->os->nextArg, cbData);
+/*@=noeffectuncon @*/
+		/* Terminate (unless explcitly continuing). */
+		if (!CBF_ISSET(cbopt, CONTINUE))
+		    return;
 	    }
-	    /* Terminate (unless explcitly continuing). */
-	    if (!(cbopt->argInfo & POPT_CBFLAG_CONTINUE))
-		return;
+	    /*@switchbreak@*/ break;
 	}
     }
 }
 
 poptContext poptGetContext(const char * name, int argc, const char ** argv,
-			   const struct poptOption * options, unsigned int flags)
+			const struct poptOption * options, unsigned int flags)
 {
     poptContext con = malloc(sizeof(*con));
 
@@ -183,7 +178,7 @@ poptContext poptGetContext(const char * name, int argc, const char ** argv,
     if (!(flags & POPT_CONTEXT_KEEP_FIRST))
 	con->os->next = 1;			/* skip argv[0] */
 
-    con->leftovers = calloc( (argc + 1), sizeof(*con->leftovers) );
+    con->leftovers = calloc( (size_t)(argc + 1), sizeof(*con->leftovers) );
 /*@-dependenttrans -assignexpose@*/	/* FIX: W2DO? */
     con->options = options;
 /*@=dependenttrans =assignexpose@*/
@@ -193,21 +188,17 @@ poptContext poptGetContext(const char * name, int argc, const char ** argv,
     con->execs = NULL;
     con->numExecs = 0;
     con->finalArgvAlloced = argc * 2;
-    con->finalArgv = calloc( con->finalArgvAlloced, sizeof(*con->finalArgv) );
+    con->finalArgv = calloc( (size_t)con->finalArgvAlloced, sizeof(*con->finalArgv) );
     con->execAbsolute = 1;
     con->arg_strip = NULL;
 
     if (getenv("POSIXLY_CORRECT") || getenv("POSIX_ME_HARDER"))
 	con->flags |= POPT_CONTEXT_POSIXMEHARDER;
 
-    if (name) {
-	char * t = malloc(strlen(name) + 1);
-	if (t) con->appName = strcpy(t, name);
-    }
+    if (name)
+	con->appName = xstrdup(name);
 
-/*@-internalglobs@*/
     invokeCallbacksPRE(con, con->options);
-/*@=internalglobs@*/
 
     return con;
 }
@@ -222,7 +213,6 @@ static void cleanOSE(/*@special@*/ struct optionStackEntry *os)
     os->argb = PBM_FREE(os->argb);
 }
 
-/*@-boundswrite@*/
 void poptResetContext(poptContext con)
 {
     int i;
@@ -255,10 +245,8 @@ void poptResetContext(poptContext con)
     return;
 /*@=nullstate@*/
 }
-/*@=boundswrite@*/
 
 /* Only one of longName, shortName should be set, not both. */
-/*@-boundswrite@*/
 static int handleExec(/*@special@*/ poptContext con,
 		/*@null@*/ const char * longName, char shortName)
 	/*@uses con->execs, con->numExecs, con->flags, con->doExec,
@@ -301,25 +289,27 @@ static int handleExec(/*@special@*/ poptContext con,
 
     i = con->finalArgvCount++;
     if (con->finalArgv != NULL)	/* XXX can't happen */
-    {	char *s  = malloc((longName ? strlen(longName) : 0) + 3);
+    {	char *s  = malloc((longName ? strlen(longName) : 0) + sizeof("--"));
 	if (s != NULL) {	/* XXX can't happen */
-	    if (longName)
-		sprintf(s, "--%s", longName);
-	    else
-		sprintf(s, "-%c", shortName);
 	    con->finalArgv[i] = s;
+	    *s++ = '-';
+	    if (longName)
+		s = stpcpy( stpcpy(s, "-"), longName);
+	    else
+		*s++ = shortName;
+	    *s = '\0';
 	} else
 	    con->finalArgv[i] = NULL;
     }
 
     return 1;
 }
-/*@=boundswrite@*/
 
 /* Only one of longName, shortName may be set at a time */
 static int handleAlias(/*@special@*/ poptContext con,
-		/*@null@*/ const char * longName, char shortName,
-		/*@exposed@*/ /*@null@*/ const char * nextCharArg)
+		/*@null@*/ const char * longName, size_t longNameLen,
+		char shortName,
+		/*@exposed@*/ /*@null@*/ const char * nextArg)
 	/*@uses con->aliases, con->numAliases, con->optionStack, con->os,
 		con->os->currAlias, con->os->currAlias->option.longName @*/
 	/*@modifies con @*/
@@ -329,9 +319,11 @@ static int handleAlias(/*@special@*/ poptContext con,
     int i;
 
     if (item) {
-	if (longName && (item->option.longName &&
-		!strcmp(longName, item->option.longName)))
+	if (longName && item->option.longName != NULL
+	 && longNameLen == strlen(item->option.longName)
+	 && !strncmp(longName, item->option.longName, longNameLen))
 	    return 0;
+	else
 	if (shortName && shortName == item->option.shortName)
 	    return 0;
     }
@@ -341,10 +333,14 @@ static int handleAlias(/*@special@*/ poptContext con,
 
     for (i = con->numAliases - 1; i >= 0; i--) {
 	item = con->aliases + i;
-	if (longName && !(item->option.longName &&
-			!strcmp(longName, item->option.longName)))
-	    continue;
-	else if (shortName != item->option.shortName)
+	if (longName) {
+	    if (item->option.longName == NULL)
+		continue;
+	    if (longNameLen != strlen(item->option.longName))
+		continue;
+	    if (strncmp(longName, item->option.longName, longNameLen))
+		continue;
+	} else if (shortName != item->option.shortName)
 	    continue;
 	break;
     }
@@ -353,10 +349,8 @@ static int handleAlias(/*@special@*/ poptContext con,
     if ((con->os - con->optionStack + 1) == POPT_OPTION_DEPTH)
 	return POPT_ERROR_OPTSTOODEEP;
 
-/*@-boundsread@*/
-    if (nextCharArg && *nextCharArg)
-	con->os->nextCharArg = nextCharArg;
-/*@=boundsread@*/
+    if (longName == NULL && nextArg != NULL && *nextArg != '\0')
+	con->os->nextCharArg = nextArg;
 
     con->os++;
     con->os->next = 0;
@@ -364,20 +358,87 @@ static int handleAlias(/*@special@*/ poptContext con,
     con->os->nextArg = NULL;
     con->os->nextCharArg = NULL;
     con->os->currAlias = con->aliases + i;
-    rc = poptDupArgv(con->os->currAlias->argc, con->os->currAlias->argv,
-		&con->os->argc, &con->os->argv);
+    {	const char ** av;
+	int ac = con->os->currAlias->argc;
+	/* Append --foo=bar arg to alias argv array (if present). */ 
+	if (longName && nextArg != NULL && *nextArg != '\0') {
+	    av = malloc((ac + 1 + 1) * sizeof(*av));
+	    if (av != NULL) {	/* XXX won't happen. */
+		for (i = 0; i < ac; i++) {
+		    av[i] = con->os->currAlias->argv[i];
+		}
+		av[ac++] = nextArg;
+		av[ac] = NULL;
+	    } else	/* XXX revert to old popt behavior if malloc fails. */
+		av = con->os->currAlias->argv;
+	} else
+	    av = con->os->currAlias->argv;
+	rc = poptDupArgv(ac, av, &con->os->argc, &con->os->argv);
+	if (av != NULL && av != con->os->currAlias->argv)
+	    free(av);
+    }
     con->os->argb = NULL;
 
     return (rc ? rc : 1);
 }
 
-/*@-bounds -boundswrite @*/
+/**
+ * Return absolute path to executable by searching PATH.
+ * @param argv0		name of executable
+ * @return		(malloc'd) absolute path to executable (or NULL)
+ */
+static /*@null@*/
+const char * findProgramPath(/*@null@*/ const char * argv0)
+	/*@*/
+{
+    char *path = NULL, *s = NULL, *se;
+    char *t = NULL;
+
+    if (argv0 == NULL) return NULL;	/* XXX can't happen */
+
+    /* If there is a / in argv[0], it has to be an absolute path. */
+    /* XXX Hmmm, why not if (argv0[0] == '/') ... instead? */
+    if (strchr(argv0, '/'))
+	return xstrdup(argv0);
+
+    if ((path = getenv("PATH")) == NULL || (path = xstrdup(path)) == NULL)
+	return NULL;
+
+    /* The return buffer in t is big enough for any path. */
+    if ((t = malloc(strlen(path) + strlen(argv0) + sizeof("/"))) != NULL)
+    for (s = path; s && *s; s = se) {
+
+	/* Snip PATH element into [s,se). */
+	if ((se = strchr(s, ':')))
+	    *se++ = '\0';
+
+	/* Append argv0 to PATH element. */
+	(void) stpcpy(stpcpy(stpcpy(t, s), "/"), argv0);
+
+	/* If file is executable, bingo! */
+	if (!access(t, X_OK))
+	    break;
+    }
+
+    /* If no executable was found in PATH, return NULL. */
+    if (!(s && *s) && t != NULL) {
+	free(t);
+	t = NULL;
+    }
+/*@-modobserver -observertrans -usedef @*/
+    if (path != NULL)
+        free(path);
+/*@=modobserver =observertrans =usedef @*/
+
+    return t;
+}
+
 static int execCommand(poptContext con)
 	/*@globals internalState @*/
 	/*@modifies internalState @*/
 {
     poptItem item = con->doExec;
-    const char ** argv = NULL;
+    poptArgv argv = NULL;
     int argc = 0;
     int rc;
     int ec = POPT_ERROR_ERRNO;
@@ -396,10 +457,11 @@ static int execCommand(poptContext con)
     if (!strchr(item->argv[0], '/') && con->execPath != NULL) {
 	char *s = malloc(strlen(con->execPath) + strlen(item->argv[0]) + sizeof("/"));
 	if (s)
-	    sprintf(s, "%s/%s", con->execPath, item->argv[0]);
+	    (void)stpcpy(stpcpy(stpcpy(s, con->execPath), "/"), item->argv[0]);
+
 	argv[argc] = s;
     } else
-	argv[argc] = POPT_findProgramPath(item->argv[0]);
+	argv[argc] = findProgramPath(item->argv[0]);
     if (argv[argc++] == NULL) {
 	ec = POPT_ERROR_NOARG;
 	goto exit;
@@ -451,7 +513,7 @@ static int execCommand(poptContext con)
 
 #ifdef	MYDEBUG
 if (_popt_debug)
-    {	const char ** avp;
+    {	poptArgv avp;
 	fprintf(stderr, "==> execvp(%s) argv[%d]:", argv[0], argc);
 	for (avp = argv; *avp; avp++)
 	    fprintf(stderr, " '%s'", *avp);
@@ -459,7 +521,9 @@ if (_popt_debug)
     }
 #endif
 
+/*@-nullstate@*/
     rc = execvp(argv[0], (char *const *)argv);
+/*@=nullstate@*/
 
 exit:
     if (argv) {
@@ -469,11 +533,10 @@ exit:
     }
     return ec;
 }
-/*@=bounds =boundswrite @*/
 
-/*@-boundswrite@*/
 /*@observer@*/ /*@null@*/ static const struct poptOption *
-findOption(const struct poptOption * opt, /*@null@*/ const char * longName, int longNameLen,
+findOption(const struct poptOption * opt,
+		/*@null@*/ const char * longName, size_t longNameLen,
 		char shortName,
 		/*@null@*/ /*@out@*/ poptCallbackType * callback,
 		/*@null@*/ /*@out@*/ const void ** callbackData,
@@ -481,24 +544,22 @@ findOption(const struct poptOption * opt, /*@null@*/ const char * longName, int
 	/*@modifies *callback, *callbackData */
 {
     const struct poptOption * cb = NULL;
+    poptArg cbarg = { .ptr = NULL };
 
     /* This happens when a single - is given */
     if (singleDash && !shortName && (longName && *longName == '\0'))
 	shortName = '-';
 
     for (; opt->longName || opt->shortName || opt->arg; opt++) {
+	poptArg arg = { .ptr = opt->arg };
 
-	if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_INCLUDE_TABLE) {
+	if (poptArgType(opt) == POPT_ARG_INCLUDE_TABLE) {
 	    const struct poptOption * opt2;
-	    void * arg = opt->arg;
 
-/*@-branchstate@*/
-	    /* XXX sick hack to preserve pretense of ABI. */
-	    if (arg == poptHelpOptions) arg = poptHelpOptionsI18N;
-/*@=branchstate@*/
+	    poptSubstituteHelpI18N(arg.opt);	/* XXX side effects */
 	    /* Recurse on included sub-tables. */
-	    if (arg == NULL) continue;	/* XXX program error */
-	    opt2 = findOption(arg, longName, longNameLen, shortName, callback,
+	    if (arg.ptr == NULL) continue;	/* XXX program error */
+	    opt2 = findOption(arg.opt, longName, longNameLen, shortName, callback,
 			      callbackData, singleDash);
 	    if (opt2 == NULL) continue;
 	    /* Sub-table data will be inheirited if no data yet. */
@@ -508,10 +569,11 @@ findOption(const struct poptOption * opt, /*@null@*/ const char * longName, int
 	    *callbackData = opt->descrip;
 /*@=observertrans =dependenttrans @*/
 	    return opt2;
-	} else if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_CALLBACK) {
+	} else if (poptArgType(opt) == POPT_ARG_CALLBACK) {
 	    cb = opt;
+	    cbarg.ptr = opt->arg;
 	} else if (longName != NULL && opt->longName != NULL &&
-		   (!singleDash || (opt->argInfo & POPT_ARGFLAG_ONEDASH)) &&
+		   (!singleDash || F_ISSET(opt, ONEDASH)) &&
 		   (!strncmp(longName, opt->longName, longNameLen) && strlen(opt->longName) == longNameLen))
 	{
 	    break;
@@ -524,25 +586,16 @@ findOption(const struct poptOption * opt, /*@null@*/ const char * longName, int
 	return NULL;
 
 /*@-modobserver -mods @*/
-    if (callback) *callback = NULL;
-    if (callbackData) *callbackData = NULL;
-    if (cb) {
-	if (callback)
-	/*@-castfcnptr@*/
-	    *callback = (poptCallbackType)cb->arg;
-	/*@=castfcnptr@*/
-	if (!(cb->argInfo & POPT_CBFLAG_INC_DATA)) {
-	    if (callbackData)
+    if (callback)
+	*callback = (cb ? cbarg.cb : NULL);
+    if (callbackData)
 /*@-observertrans@*/	/* FIX: typedef double indirection. */
-		*callbackData = cb->descrip;
+	*callbackData = (cb && !CBF_ISSET(cb, INC_DATA) ? cb->descrip : NULL);
 /*@=observertrans@*/
-	}
-    }
 /*@=modobserver =mods @*/
 
     return opt;
 }
-/*@=boundswrite@*/
 
 static const char * findNextArg(/*@special@*/ poptContext con,
 		unsigned argx, int delete_arg)
@@ -581,7 +634,6 @@ static const char * findNextArg(/*@special@*/ poptContext con,
     return arg;
 }
 
-/*@-boundswrite@*/
 static /*@only@*/ /*@null@*/ const char *
 expandNextArg(/*@special@*/ poptContext con, const char * s)
 	/*@uses con->optionStack, con->os,
@@ -589,13 +641,13 @@ expandNextArg(/*@special@*/ poptContext con, const char * s)
 	/*@modifies con @*/
 {
     const char * a = NULL;
-    size_t alen;
     char *t, *te;
     size_t tn = strlen(s) + 1;
     char c;
 
-    te = t = malloc(tn);;
+    te = t = malloc(tn);
     if (t == NULL) return NULL;		/* XXX can't happen */
+    *t = '\0';
     while ((c = *s++) != '\0') {
 	switch (c) {
 #if 0	/* XXX can't do this */
@@ -613,12 +665,11 @@ expandNextArg(/*@special@*/ poptContext con, const char * s)
 	    }
 	    s += 3;
 
-	    alen = strlen(a);
-	    tn += alen;
-	    *te = '\0';
-	    t = realloc(t, tn);
-	    te = t + strlen(t);
-	    strncpy(te, a, alen); te += alen;
+	    tn += strlen(a);
+	    {   size_t pos = (size_t) (te - t);
+		t = realloc(t, tn);
+		te = stpcpy(t + pos, a);
+	    }
 	    continue;
 	    /*@notreached@*/ /*@switchbreak@*/ break;
 	default:
@@ -626,39 +677,104 @@ expandNextArg(/*@special@*/ poptContext con, const char * s)
 	}
 	*te++ = c;
     }
-    *te = '\0';
-    t = realloc(t, strlen(t) + 1);	/* XXX memory leak, hard to plug */
+    *te++ = '\0';
+    /* If the new string is longer than needed, shorten. */
+    if ((t + tn) > te) {
+/*@-usereleased@*/	/* XXX splint can't follow the pointers. */
+	if ((te = realloc(t, (size_t)(te - t))) == NULL)
+	    free(t);
+	t = te;
+/*@=usereleased@*/
+    }
     return t;
 }
-/*@=boundswrite@*/
 
 static void poptStripArg(/*@special@*/ poptContext con, int which)
-	/*@uses con->arg_strip, con->optionStack @*/
+	/*@uses con->optionStack @*/
 	/*@defines con->arg_strip @*/
 	/*@modifies con @*/
 {
-/*@-sizeoftype@*/
+/*@-compdef -sizeoftype -usedef @*/
     if (con->arg_strip == NULL)
 	con->arg_strip = PBM_ALLOC(con->optionStack[0].argc);
     if (con->arg_strip != NULL)		/* XXX can't happen */
     PBM_SET(which, con->arg_strip);
-/*@=sizeoftype@*/
-/*@-compdef@*/ /* LCL: con->arg_strip undefined? */
     return;
-/*@=compdef@*/
+/*@=compdef =sizeoftype =usedef @*/
+}
+
+int poptSaveString(const char *** argvp,
+		/*@unused@*/ UNUSED(unsigned int argInfo), const char * val)
+{
+    int argc = 0;
+
+    if (argvp == NULL)
+	return -1;
+
+    /* XXX likely needs an upper bound on argc. */
+    if (*argvp != NULL)
+    while ((*argvp)[argc] != NULL)
+	argc++;
+ 
+/*@-unqualifiedtrans -nullstate@*/	/* XXX no annotation for (*argvp) */
+    if ((*argvp = xrealloc(*argvp, (argc + 1 + 1) * sizeof(**argvp))) != NULL) {
+	(*argvp)[argc++] = xstrdup(val);
+	(*argvp)[argc  ] = NULL;
+    }
+    return 0;
+/*@=unqualifiedtrans =nullstate@*/
 }
 
 /*@unchecked@*/
 static unsigned int seed = 0;
 
-/*@-bitwisesigned@*/	/* LCL: logical ops with unsigned. */
+int poptSaveLongLong(long long * arg, unsigned int argInfo, long long aLongLong)
+{
+    if (arg == NULL
+#ifdef	NOTYET
+    /* XXX Check alignment, may fail on funky platforms. */
+     || (((unsigned long long)arg) & (sizeof(*arg)-1))
+#endif
+    )
+	return POPT_ERROR_NULLARG;
+
+    if (aLongLong != 0 && LF_ISSET(RANDOM)) {
+	if (!seed) {
+	    srandom((unsigned)getpid());
+	    srandom((unsigned)random());
+	}
+	aLongLong = (long long)(random() % (aLongLong > 0 ? aLongLong : -aLongLong));
+	aLongLong++;
+    }
+    if (LF_ISSET(NOT))
+	aLongLong = ~aLongLong;
+    switch (LF_ISSET(LOGICALOPS)) {
+    case 0:
+	*arg = aLongLong;
+	break;
+    case POPT_ARGFLAG_OR:
+	*(unsigned long long *)arg |= (unsigned long long)aLongLong;
+	break;
+    case POPT_ARGFLAG_AND:
+	*(unsigned long long *)arg &= (unsigned long long)aLongLong;
+	break;
+    case POPT_ARGFLAG_XOR:
+	*(unsigned long long *)arg ^= (unsigned long long)aLongLong;
+	break;
+    default:
+	return POPT_ERROR_BADOPERATION;
+	/*@notreached@*/ break;
+    }
+    return 0;
+}
+
 int poptSaveLong(long * arg, unsigned int argInfo, long aLong)
 {
     /* XXX Check alignment, may fail on funky platforms. */
     if (arg == NULL || (((unsigned long)arg) & (sizeof(*arg)-1)))
 	return POPT_ERROR_NULLARG;
 
-    if (aLong != 0 && argInfo & POPT_ARGFLAG_RANDOM) {
+    if (aLong != 0 && LF_ISSET(RANDOM)) {
 	if (!seed) {
 	    srandom((unsigned)getpid());
 	    srandom((unsigned)random());
@@ -666,20 +782,20 @@ int poptSaveLong(long * arg, unsigned int argInfo, long aLong)
 	aLong = random() % (aLong > 0 ? aLong : -aLong);
 	aLong++;
     }
-    if (argInfo & POPT_ARGFLAG_NOT)
+    if (LF_ISSET(NOT))
 	aLong = ~aLong;
-    switch (argInfo & POPT_ARGFLAG_LOGICALOPS) {
+    switch (LF_ISSET(LOGICALOPS)) {
     case 0:
 	*arg = aLong;
 	break;
     case POPT_ARGFLAG_OR:
-	*arg |= aLong;
+	*(unsigned long *)arg |= (unsigned long)aLong;
 	break;
     case POPT_ARGFLAG_AND:
-	*arg &= aLong;
+	*(unsigned long *)arg &= (unsigned long)aLong;
 	break;
     case POPT_ARGFLAG_XOR:
-	*arg ^= aLong;
+	*(unsigned long *)arg ^= (unsigned long)aLong;
 	break;
     default:
 	return POPT_ERROR_BADOPERATION;
@@ -687,16 +803,14 @@ int poptSaveLong(long * arg, unsigned int argInfo, long aLong)
     }
     return 0;
 }
-/*@=bitwisesigned@*/
 
-/*@-bitwisesigned@*/	/* LCL: logical ops with unsigned. */
 int poptSaveInt(/*@null@*/ int * arg, unsigned int argInfo, long aLong)
 {
     /* XXX Check alignment, may fail on funky platforms. */
     if (arg == NULL || (((unsigned long)arg) & (sizeof(*arg)-1)))
 	return POPT_ERROR_NULLARG;
 
-    if (aLong != 0 && argInfo & POPT_ARGFLAG_RANDOM) {
+    if (aLong != 0 && LF_ISSET(RANDOM)) {
 	if (!seed) {
 	    srandom((unsigned)getpid());
 	    srandom((unsigned)random());
@@ -704,20 +818,20 @@ int poptSaveInt(/*@null@*/ int * arg, unsigned int argInfo, long aLong)
 	aLong = random() % (aLong > 0 ? aLong : -aLong);
 	aLong++;
     }
-    if (argInfo & POPT_ARGFLAG_NOT)
+    if (LF_ISSET(NOT))
 	aLong = ~aLong;
-    switch (argInfo & POPT_ARGFLAG_LOGICALOPS) {
+    switch (LF_ISSET(LOGICALOPS)) {
     case 0:
-	*arg = aLong;
+	*arg = (int) aLong;
 	break;
     case POPT_ARGFLAG_OR:
-	*arg |= aLong;
+	*(unsigned int *)arg |= (unsigned int) aLong;
 	break;
     case POPT_ARGFLAG_AND:
-	*arg &= aLong;
+	*(unsigned int *)arg &= (unsigned int) aLong;
 	break;
     case POPT_ARGFLAG_XOR:
-	*arg ^= aLong;
+	*(unsigned int *)arg ^= (unsigned int) aLong;
 	break;
     default:
 	return POPT_ERROR_BADOPERATION;
@@ -725,9 +839,7 @@ int poptSaveInt(/*@null@*/ int * arg, unsigned int argInfo, long aLong)
     }
     return 0;
 }
-/*@=bitwisesigned@*/
 
-/*@-boundswrite@*/
 /* returns 'val' element, -1 on last item, POPT_ERROR_* on error */
 int poptGetNextOpt(poptContext con)
 {
@@ -765,7 +877,7 @@ int poptGetNextOpt(poptContext con)
 	/* Process next long option */
 	if (!con->os->nextCharArg) {
 	    const char * optString;
-            int optStringLen;
+            size_t optStringLen;
 	    int thisopt;
 
 /*@-sizeoftype@*/
@@ -814,20 +926,22 @@ int poptGetNextOpt(poptContext con)
 		else
 		    singleDash = 1;
 
-		/* XXX aliases with arg substitution need "--alias=arg" */
-		if (handleAlias(con, optString, '\0', NULL))
-		    continue;
-
-		if (handleExec(con, optString, '\0'))
-		    continue;
-
 		/* Check for "--long=arg" option. */
 		for (oe = optString; *oe && *oe != '='; oe++)
 		    {};
-		optStringLen = oe - optString;
+		optStringLen = (size_t)(oe - optString);
 		if (*oe == '=')
 		    longArg = oe + 1;
 
+		/* XXX aliases with arg substitution need "--alias=arg" */
+		if (handleAlias(con, optString, optStringLen, '\0', longArg)) {
+		    longArg = NULL;
+		    continue;
+		}
+
+		if (handleExec(con, optString, '\0'))
+		    continue;
+
 		opt = findOption(con->options, optString, optStringLen, '\0', &cb, &cbData,
 				 singleDash);
 		if (!opt && !singleDash)
@@ -836,9 +950,9 @@ int poptGetNextOpt(poptContext con)
 
 	    if (!opt) {
 		con->os->nextCharArg = origOptString + 1;
+		longArg = NULL;
 	    } else {
-		if (con->os == con->optionStack &&
-		   opt->argInfo & POPT_ARGFLAG_STRIP)
+		if (con->os == con->optionStack && F_ISSET(opt, STRIP))
 		{
 		    canstrip = 1;
 		    poptStripArg(con, thisopt);
@@ -848,13 +962,12 @@ int poptGetNextOpt(poptContext con)
 	}
 
 	/* Process next short option */
-/*@-branchstate@*/		/* FIX: W2DO? */
 	if (con->os->nextCharArg) {
 	    origOptString = con->os->nextCharArg;
 
 	    con->os->nextCharArg = NULL;
 
-	    if (handleAlias(con, NULL, *origOptString, origOptString + 1))
+	    if (handleAlias(con, NULL, 0, *origOptString, origOptString + 1))
 		continue;
 
 	    if (handleExec(con, NULL, *origOptString)) {
@@ -873,35 +986,35 @@ int poptGetNextOpt(poptContext con)
 
 	    origOptString++;
 	    if (*origOptString != '\0')
-		con->os->nextCharArg = origOptString;
+		con->os->nextCharArg = origOptString + (int)(*origOptString == '=');
 	}
-/*@=branchstate@*/
 
 	if (opt == NULL) return POPT_ERROR_BADOPT;	/* XXX can't happen */
-	if (opt->arg && (opt->argInfo & POPT_ARG_MASK) == POPT_ARG_NONE) {
+	if (opt->arg && poptArgType(opt) == POPT_ARG_NONE) {
 	    if (poptSaveInt((int *)opt->arg, opt->argInfo, 1L))
 		return POPT_ERROR_BADOPERATION;
-	} else if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_VAL) {
+	} else if (poptArgType(opt) == POPT_ARG_VAL) {
 	    if (opt->arg) {
 		if (poptSaveInt((int *)opt->arg, opt->argInfo, (long)opt->val))
 		    return POPT_ERROR_BADOPERATION;
 	    }
-	} else if ((opt->argInfo & POPT_ARG_MASK) != POPT_ARG_NONE) {
+	} else if (poptArgType(opt) != POPT_ARG_NONE) {
 	    con->os->nextArg = _free(con->os->nextArg);
 	    if (longArg) {
 		longArg = expandNextArg(con, longArg);
-		con->os->nextArg = longArg;
+		con->os->nextArg = (char *) longArg;
 	    } else if (con->os->nextCharArg) {
 		longArg = expandNextArg(con, con->os->nextCharArg);
-		con->os->nextArg = longArg;
+		con->os->nextArg = (char *) longArg;
 		con->os->nextCharArg = NULL;
 	    } else {
 		while (con->os->next == con->os->argc &&
-		       con->os > con->optionStack) {
+			con->os > con->optionStack)
+		{
 		    cleanOSE(con->os--);
 		}
 		if (con->os->next == con->os->argc) {
-		    if (!(opt->argInfo & POPT_ARGFLAG_OPTIONAL))
+		    if (!F_ISSET(opt, OPTIONAL))
 			return POPT_ERROR_NOARG;
 		    con->os->nextArg = NULL;
 		} else {
@@ -910,21 +1023,21 @@ int poptGetNextOpt(poptContext con)
 		     * Make sure this isn't part of a short arg or the
 		     * result of an alias expansion.
 		     */
-		    if (con->os == con->optionStack &&
-			(opt->argInfo & POPT_ARGFLAG_STRIP) &&
-			canstrip) {
+		    if (con->os == con->optionStack
+		     && F_ISSET(opt, STRIP) && canstrip)
+		    {
 			poptStripArg(con, con->os->next);
 		    }
 		
 		    if (con->os->argv != NULL) {	/* XXX can't happen */
-			if (opt->argInfo & POPT_ARGFLAG_OPTIONAL &&
+			if (F_ISSET(opt, OPTIONAL) &&
 			    con->os->argv[con->os->next][0] == '-') {
 			    con->os->nextArg = NULL;
 			} else {
 			    /* XXX watchout: subtle side-effects live here. */
 			    longArg = con->os->argv[con->os->next++];
 			    longArg = expandNextArg(con, longArg);
-			    con->os->nextArg = longArg;
+			    con->os->nextArg = (char *) longArg;
 			}
 		    }
 		}
@@ -932,35 +1045,58 @@ int poptGetNextOpt(poptContext con)
 	    longArg = NULL;
 
 	    if (opt->arg) {
-		switch (opt->argInfo & POPT_ARG_MASK) {
+		poptArg arg = { .ptr = opt->arg };
+		switch (poptArgType(opt)) {
+		case POPT_ARG_ARGV:
+		    /* XXX memory leak, application is responsible for free. */
+		    if (con->os->nextArg == NULL)
+			return POPT_ERROR_NULLARG;	/* XXX better return? */
+		    if (poptSaveString(arg.ptr, opt->argInfo, con->os->nextArg))
+			return POPT_ERROR_BADOPERATION;
+		    /*@switchbreak@*/ break;
 		case POPT_ARG_STRING:
-		    /* XXX memory leak, hard to plug */
-		    *((const char **) opt->arg) = (con->os->nextArg)
+		    /* XXX memory leak, application is responsible for free. */
+		    arg.argv[0] = (con->os->nextArg)
 			? xstrdup(con->os->nextArg) : NULL;
 		    /*@switchbreak@*/ break;
 
 		case POPT_ARG_INT:
 		case POPT_ARG_LONG:
-		{   long aLong = 0;
-		    char *end;
+		case POPT_ARG_LONGLONG:
+		{   long long aNUM = 0;
+		    char *end = NULL;
 
 		    if (con->os->nextArg) {
-			aLong = strtol(con->os->nextArg, &end, 0);
+			aNUM = strtoll(con->os->nextArg, &end, 0);
 			if (!(end && *end == '\0'))
 			    return POPT_ERROR_BADNUMBER;
 		    }
 
-		    if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_LONG) {
-			if (aLong == LONG_MIN || aLong == LONG_MAX)
+/* XXX let's not demand C99 compiler flags for <limits.h> quite yet. */
+#if !defined(LLONG_MAX)
+#   define LLONG_MAX    9223372036854775807LL
+#   define LLONG_MIN    (-LLONG_MAX - 1LL)
+#endif
+
+		    if (poptArgType(opt) == POPT_ARG_LONGLONG) {
+			if (aNUM == LLONG_MAX || aNUM == LLONG_MIN)
 			    return POPT_ERROR_OVERFLOW;
-			if (poptSaveLong((long *)opt->arg, opt->argInfo, aLong))
+			if (poptSaveLongLong(arg.longlongp, opt->argInfo, aNUM))
 			    return POPT_ERROR_BADOPERATION;
-		    } else {
-			if (aLong > INT_MAX || aLong < INT_MIN)
+		    } else
+		    if (poptArgType(opt) == POPT_ARG_LONG) {
+			if (aNUM > (long long)LONG_MAX || aNUM < (long long)LONG_MIN)
 			    return POPT_ERROR_OVERFLOW;
-			if (poptSaveInt((int *)opt->arg, opt->argInfo, aLong))
+			if (poptSaveLong(arg.longp, opt->argInfo, (long)aNUM))
 			    return POPT_ERROR_BADOPERATION;
-		    }
+		    } else
+		    if (poptArgType(opt) == POPT_ARG_INT) {
+			if (aNUM > (long long)INT_MAX || aNUM < (long long)INT_MIN)
+			    return POPT_ERROR_OVERFLOW;
+			if (poptSaveInt(arg.intp, opt->argInfo, (long)aNUM))
+			    return POPT_ERROR_BADOPERATION;
+		    } else
+			return POPT_ERROR_BADOPERATION;
 		}   /*@switchbreak@*/ break;
 
 		case POPT_ARG_FLOAT:
@@ -981,15 +1117,18 @@ int poptGetNextOpt(poptContext con)
 			    return POPT_ERROR_BADNUMBER;
 		    }
 
-		    if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_DOUBLE) {
-			*((double *) opt->arg) = aDouble;
+		    if (poptArgType(opt) == POPT_ARG_DOUBLE) {
+			arg.doublep[0] = aDouble;
 		    } else {
-#define _ABS(a)	((((a) - 0.0) < DBL_EPSILON) ? -(a) : (a))
-			if ((_ABS(aDouble) - FLT_MAX) > DBL_EPSILON)
+#if !defined(DBL_EPSILON) && !defined(__LCLINT__)
+#define DBL_EPSILON 2.2204460492503131e-16
+#endif
+#define POPT_ABS(a)	((((a) - 0.0) < DBL_EPSILON) ? -(a) : (a))
+			if ((POPT_ABS(aDouble) - FLT_MAX) > DBL_EPSILON)
 			    return POPT_ERROR_OVERFLOW;
-			if ((FLT_MIN - _ABS(aDouble)) > DBL_EPSILON)
+			if ((FLT_MIN - POPT_ABS(aDouble)) > DBL_EPSILON)
 			    return POPT_ERROR_OVERFLOW;
-			*((float *) opt->arg) = aDouble;
+			arg.floatp[0] = (float) aDouble;
 		    }
 		}   /*@switchbreak@*/ break;
 		case POPT_ARG_MAINCALL:
@@ -999,8 +1138,8 @@ int poptGetNextOpt(poptContext con)
 		    /*@switchbreak@*/ break;
 		default:
 		    fprintf(stdout,
-			POPT_("option type (%d) not implemented in popt\n"),
-			(opt->argInfo & POPT_ARG_MASK));
+			POPT_("option type (%u) not implemented in popt\n"),
+			poptArgType(opt));
 		    exit(EXIT_FAILURE);
 		    /*@notreached@*/ /*@switchbreak@*/ break;
 		}
@@ -1009,7 +1148,7 @@ int poptGetNextOpt(poptContext con)
 
 	if (cb)
 	    invokeCallbacksOPTION(con, con->options, opt, cbData, shorty);
-	else if (opt->val && ((opt->argInfo & POPT_ARG_MASK) != POPT_ARG_VAL))
+	else if (opt->val && (poptArgType(opt) != POPT_ARG_VAL))
 	    done = 1;
 
 	if ((con->finalArgvCount + 2) >= (con->finalArgvAlloced)) {
@@ -1019,24 +1158,27 @@ int poptGetNextOpt(poptContext con)
 	}
 
 	if (con->finalArgv != NULL)
-	{   char *s = malloc((opt->longName ? strlen(opt->longName) : 0) + 3);
+	{   char *s = malloc((opt->longName ? strlen(opt->longName) : 0) + sizeof("--"));
 	    if (s != NULL) {	/* XXX can't happen */
-		if (opt->longName)
-		    sprintf(s, "%s%s",
-			((opt->argInfo & POPT_ARGFLAG_ONEDASH) ? "-" : "--"),
-			opt->longName);
-		else
-		    sprintf(s, "-%c", opt->shortName);
 		con->finalArgv[con->finalArgvCount++] = s;
+		*s++ = '-';
+		if (opt->longName) {
+		    if (!F_ISSET(opt, ONEDASH))
+			*s++ = '-';
+		    s = stpcpy(s, opt->longName);
+		} else {
+		    *s++ = opt->shortName;
+		    *s = '\0';
+		}
 	    } else
 		con->finalArgv[con->finalArgvCount++] = NULL;
 	}
 
-	if (opt->arg && (opt->argInfo & POPT_ARG_MASK) == POPT_ARG_NONE)
+	if (opt->arg && poptArgType(opt) == POPT_ARG_NONE)
 	    /*@-ifempty@*/ ; /*@=ifempty@*/
-	else if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_VAL)
+	else if (poptArgType(opt) == POPT_ARG_VAL)
 	    /*@-ifempty@*/ ; /*@=ifempty@*/
-	else if ((opt->argInfo & POPT_ARG_MASK) != POPT_ARG_NONE) {
+	else if (poptArgType(opt) != POPT_ARG_NONE) {
 	    if (con->finalArgv != NULL && con->os->nextArg != NULL)
 	        con->finalArgv[con->finalArgvCount++] =
 			xstrdup(con->os->nextArg);
@@ -1045,17 +1187,14 @@ int poptGetNextOpt(poptContext con)
 
     return (opt ? opt->val : -1);	/* XXX can't happen */
 }
-/*@=boundswrite@*/
 
-const char * poptGetOptArg(poptContext con)
+char * poptGetOptArg(poptContext con)
 {
-    const char * ret = NULL;
-/*@-branchstate@*/
+    char * ret = NULL;
     if (con) {
 	ret = con->os->nextArg;
 	con->os->nextArg = NULL;
     }
-/*@=branchstate@*/
     return ret;
 }
 
@@ -1075,7 +1214,6 @@ const char * poptPeekArg(poptContext con)
     return ret;
 }
 
-/*@-boundswrite@*/
 const char ** poptGetArgs(poptContext con)
 {
     if (con == NULL ||
@@ -1089,40 +1227,38 @@ const char ** poptGetArgs(poptContext con)
     return (con->leftovers + con->nextLeftover);
 /*@=nullret =nullstate @*/
 }
-/*@=boundswrite@*/
+
+static /*@null@*/
+poptItem poptFreeItems(/*@only@*/ /*@null@*/ poptItem items, int nitems)
+	/*@modifies items @*/
+{
+    if (items != NULL) {
+	poptItem item = items;
+	while (--nitems >= 0) {
+/*@-modobserver -observertrans -dependenttrans@*/
+	    item->option.longName = _free(item->option.longName);
+	    item->option.descrip = _free(item->option.descrip);
+	    item->option.argDescrip = _free(item->option.argDescrip);
+/*@=modobserver =observertrans =dependenttrans@*/
+	    item->argv = _free(item->argv);
+	    item++;
+	}
+	items = _free(items);
+    }
+    return NULL;
+}
 
 poptContext poptFreeContext(poptContext con)
 {
-    poptItem item;
-    int i;
-
     if (con == NULL) return con;
     poptResetContext(con);
     con->os->argb = _free(con->os->argb);
 
-    if (con->aliases != NULL)
-    for (i = 0; i < con->numAliases; i++) {
-	item = con->aliases + i;
-/*@-modobserver -observertrans -dependenttrans@*/
-	item->option.longName = _free(item->option.longName);
-	item->option.descrip = _free(item->option.descrip);
-	item->option.argDescrip = _free(item->option.argDescrip);
-/*@=modobserver =observertrans =dependenttrans@*/
-	item->argv = _free(item->argv);
-    }
-    con->aliases = _free(con->aliases);
+    con->aliases = poptFreeItems(con->aliases, con->numAliases);
+    con->numAliases = 0;
 
-    if (con->execs != NULL)
-    for (i = 0; i < con->numExecs; i++) {
-	item = con->execs + i;
-/*@-modobserver -observertrans -dependenttrans@*/
-	item->option.longName = _free(item->option.longName);
-	item->option.descrip = _free(item->option.descrip);
-	item->option.argDescrip = _free(item->option.argDescrip);
-/*@=modobserver =observertrans =dependenttrans@*/
-	item->argv = _free(item->argv);
-    }
-    con->execs = _free(con->execs);
+    con->execs = poptFreeItems(con->execs, con->numExecs);
+    con->numExecs = 0;
 
     con->leftovers = _free(con->leftovers);
     con->finalArgv = _free(con->finalArgv);
@@ -1136,7 +1272,7 @@ poptContext poptFreeContext(poptContext con)
 }
 
 int poptAddAlias(poptContext con, struct poptAlias alias,
-		/*@unused@*/ int flags)
+		/*@unused@*/ UNUSED(int flags))
 {
     struct poptItem_s item_buf;
     poptItem item = &item_buf;
@@ -1153,7 +1289,6 @@ int poptAddAlias(poptContext con, struct poptAlias alias,
     return poptAddItem(con, item, 0);
 }
 
-/*@-boundswrite@*/
 int poptAddItem(poptContext con, poptItem newItem, int flags)
 {
     poptItem * items, item;
@@ -1196,7 +1331,6 @@ int poptAddItem(poptContext con, poptItem newItem, int flags)
 
     return 0;
 }
-/*@=boundswrite@*/
 
 const char * poptBadOption(poptContext con, unsigned int flags)
 {
@@ -1264,7 +1398,6 @@ const char * poptGetInvocationName(poptContext con)
     return (con->os->argv ? con->os->argv[0] : "");
 }
 
-/*@-boundswrite@*/
 int poptStrippedArgv(poptContext con, int argc, char ** argv)
 {
     int numargs = argc;
@@ -1288,4 +1421,3 @@ int poptStrippedArgv(poptContext con, int argc, char ** argv)
     
     return numargs;
 }
-/*@=boundswrite@*/
