@@ -10,6 +10,11 @@
 
 #include "system.h"
 
+#define        POPT_USE_TIOCGWINSZ
+#ifdef POPT_USE_TIOCGWINSZ
+#include <sys/ioctl.h>
+#endif
+
 #define	POPT_WCHAR_HACK
 #ifdef 	POPT_WCHAR_HACK
 #include <wchar.h>			/* for mbsrtowcs */
@@ -19,8 +24,6 @@
 
 /*@access poptContext@*/
 
-#define	_POPTHELP_MAXLINE	((size_t)79)
-
 /**
  * Display arguments.
  * @param con		context
@@ -31,17 +34,21 @@
  */
 /*@exits@*/
 static void displayArgs(poptContext con,
-		/*@unused@*/ enum poptCallbackReason foo,
+		/*@unused@*/ UNUSED(enum poptCallbackReason foo),
 		struct poptOption * key, 
-		/*@unused@*/ const char * arg, /*@unused@*/ void * data)
+		/*@unused@*/ UNUSED(const char * arg),
+		/*@unused@*/ UNUSED(void * data))
 	/*@globals fileSystem@*/
-	/*@modifies con, fileSystem@*/
+	/*@modifies fileSystem@*/
 {
     if (key->shortName == '?')
 	poptPrintHelp(con, stdout, 0);
     else
 	poptPrintUsage(con, stdout, 0);
-/*@i@*/ con = poptFreeContext(con);		/* XXX keep valgrind happy */
+
+#if !defined(__LCLINT__)	/* XXX keep both splint & valgrind happy */
+    con = poptFreeContext(con);
+#endif
     exit(0);
 }
 
@@ -82,6 +89,7 @@ static struct poptOption poptHelpOptions2[] = {
   { "defaults", '\0', POPT_ARG_NONE, &show_option_defaults, 0,
 	N_("Display option defaults in message"), NULL },
 #endif
+  { "", '\0',	0, NULL, 0, N_("Terminate options"), NULL },
     POPT_TABLEEND
 } ;
 
@@ -89,17 +97,69 @@ static struct poptOption poptHelpOptions2[] = {
 struct poptOption * poptHelpOptionsI18N = poptHelpOptions2;
 /*@=castfcnptr@*/
 
+#define        _POPTHELP_MAXLINE       ((size_t)79)
+
+typedef struct columns_s {
+    size_t cur;
+    size_t max;
+} * columns_t;
+
+/** 
+ * Return no. of columns in output window.
+ * @param fp           FILE
+ * @return             no. of columns 
+ */ 
+static size_t maxColumnWidth(FILE *fp)
+	/*@*/
+{   
+    size_t maxcols = _POPTHELP_MAXLINE;
+#if defined(TIOCGWINSZ)
+    struct winsize ws;
+    int fdno = fileno(fp ? fp : stdout);
+
+    memset(&ws, 0, sizeof(ws));
+    if (fdno >= 0 && !ioctl(fdno, (unsigned long)TIOCGWINSZ, &ws)) {
+	size_t ws_col = (size_t)ws.ws_col;
+	if (ws_col > maxcols && ws_col < (size_t)256)
+	    maxcols = ws_col - 1;
+    }
+#endif
+    return maxcols;
+}   
+
+/**
+ * Determine number of display characters in a string.
+ * @param s		string
+ * @return		no. of display characters.
+ */
+static inline size_t stringDisplayWidth(const char *s)
+	/*@*/
+{
+    size_t n = strlen(s);
+#ifdef	POPT_WCHAR_HACK
+    mbstate_t t;
+
+    memset ((void *)&t, 0, sizeof (t));	/* In initial state.  */
+    /* Determine number of display characters.  */
+    n = mbsrtowcs (NULL, &s, n, &t);
+#else
+    n = 0;
+    for (; *s; s = POPT_next_char(s));
+	n++;
+#endif
+
+    return n;
+}
+
 /**
  * @param table		option(s)
  */
 /*@observer@*/ /*@null@*/ static const char *
-getTableTranslationDomain(/*@null@*/ const struct poptOption *table)
+getTableTranslationDomain(/*@null@*/ const struct poptOption *opt)
 	/*@*/
 {
-    const struct poptOption *opt;
-
-    if (table != NULL)
-    for (opt = table; opt->longName || opt->shortName || opt->arg; opt++) {
+    if (opt != NULL)
+    for (; opt->longName || opt->shortName || opt->arg; opt++) {
 	if (opt->argInfo == POPT_ARG_INTL_DOMAIN)
 	    return opt->arg;
     }
@@ -117,14 +177,26 @@ getArgDescrip(const struct poptOption * opt,
 		/*@=paramuse@*/
 	/*@*/
 {
-    if (!(opt->argInfo & POPT_ARG_MASK)) return NULL;
+    if (!poptArgType(opt)) return NULL;
 
-    if (opt == (poptHelpOptions + 1) || opt == (poptHelpOptions + 2))
-	if (opt->argDescrip) return POPT_(opt->argDescrip);
+    if (poptArgType(opt) == POPT_ARG_MAINCALL)
+	return opt->argDescrip;
+    if (poptArgType(opt) == POPT_ARG_ARGV)
+	return opt->argDescrip;
 
-    if (opt->argDescrip) return D_(translation_domain, opt->argDescrip);
+    if (opt->argDescrip) {
+	/* Some strings need popt library, not application, i18n domain. */
+	if (opt == (poptHelpOptions + 1)
+	 || opt == (poptHelpOptions + 2)
+	 || !strcmp(opt->argDescrip, N_("Help options:"))
+	 || !strcmp(opt->argDescrip, N_("Options implemented via popt alias/exec:")))
+	    return POPT_(opt->argDescrip);
 
-    switch (opt->argInfo & POPT_ARG_MASK) {
+	/* Use the application i18n domain. */
+	return D_(translation_domain, opt->argDescrip);
+    }
+
+    switch (poptArgType(opt)) {
     case POPT_ARG_NONE:		return POPT_("NONE");
 #ifdef	DYING
     case POPT_ARG_VAL:		return POPT_("VAL");
@@ -133,9 +205,12 @@ getArgDescrip(const struct poptOption * opt,
 #endif
     case POPT_ARG_INT:		return POPT_("INT");
     case POPT_ARG_LONG:		return POPT_("LONG");
+    case POPT_ARG_LONGLONG:	return POPT_("LONGLONG");
     case POPT_ARG_STRING:	return POPT_("STRING");
     case POPT_ARG_FLOAT:	return POPT_("FLOAT");
     case POPT_ARG_DOUBLE:	return POPT_("DOUBLE");
+    case POPT_ARG_MAINCALL:	return NULL;
+    case POPT_ARG_ARGV:		return NULL;
     default:			return POPT_("ARG");
     }
 }
@@ -160,42 +235,48 @@ singleOptionDefaultValue(size_t lineLength,
     char * l = le;
 
     if (le == NULL) return NULL;	/* XXX can't happen */
-/*@-boundswrite@*/
     *le = '\0';
     *le++ = '(';
-    strcpy(le, defstr);	le += strlen(le);
+    le = stpcpy(le, defstr);
     *le++ = ':';
     *le++ = ' ';
-    if (opt->arg)	/* XXX programmer error */
-    switch (opt->argInfo & POPT_ARG_MASK) {
+  if (opt->arg) {	/* XXX programmer error */
+    poptArg arg = { .ptr = opt->arg };
+    switch (poptArgType(opt)) {
     case POPT_ARG_VAL:
     case POPT_ARG_INT:
-    {	long aLong = *((int *)opt->arg);
-	le += sprintf(le, "%ld", aLong);
-    }	break;
+	le += sprintf(le, "%d", arg.intp[0]);
+	break;
     case POPT_ARG_LONG:
-    {	long aLong = *((long *)opt->arg);
-	le += sprintf(le, "%ld", aLong);
-    }	break;
+	le += sprintf(le, "%ld", arg.longp[0]);
+	break;
+    case POPT_ARG_LONGLONG:
+	le += sprintf(le, "%lld", arg.longlongp[0]);
+	break;
     case POPT_ARG_FLOAT:
-    {	double aDouble = *((float *)opt->arg);
+    {	double aDouble = (double) arg.floatp[0];
 	le += sprintf(le, "%g", aDouble);
     }	break;
     case POPT_ARG_DOUBLE:
-    {	double aDouble = *((double *)opt->arg);
-	le += sprintf(le, "%g", aDouble);
-    }	break;
+	le += sprintf(le, "%g", arg.doublep[0]);
+	break;
+    case POPT_ARG_MAINCALL:
+	le += sprintf(le, "%p", opt->arg);
+	break;
+    case POPT_ARG_ARGV:
+	le += sprintf(le, "%p", opt->arg);
+	break;
     case POPT_ARG_STRING:
-    {	const char * s = *(const char **)opt->arg;
-	if (s == NULL) {
-	    strcpy(le, "null");	le += strlen(le);
-	} else {
-	    size_t slen = 4*lineLength - (le - l) - sizeof("\"...\")");
+    {	const char * s = arg.argv[0];
+	if (s == NULL)
+	    le = stpcpy(le, "null");
+	else {
+	    size_t limit = 4*lineLength - (le - l) - sizeof("\"\")");
+	    size_t slen;
 	    *le++ = '"';
-	    strncpy(le, s, slen); le[slen] = '\0'; le += strlen(le);	
-	    if (slen < strlen(s)) {
-		strcpy(le, "...");	le += strlen(le);
-	    }
+	    strncpy(le, s, limit); le[limit] = '\0'; le += (slen = strlen(le));
+	    if (slen == limit && s[limit])
+		le[-1] = le[-2] = le[-3] = '.';
 	    *le++ = '"';
 	}
     }	break;
@@ -205,9 +286,9 @@ singleOptionDefaultValue(size_t lineLength,
 	return NULL;
 	/*@notreached@*/ break;
     }
+  }
     *le++ = ')';
     *le = '\0';
-/*@=boundswrite@*/
 
     return l;
 }
@@ -215,67 +296,75 @@ singleOptionDefaultValue(size_t lineLength,
 /**
  * Display help text for an option.
  * @param fp		output file handle
- * @param maxLeftCol	largest argument display width
+ * @param columns	output display width control
  * @param opt		option(s)
  * @param translation_domain	translation domain
  */
-static void singleOptionHelp(FILE * fp, size_t maxLeftCol, 
+static void singleOptionHelp(FILE * fp, columns_t columns,
 		const struct poptOption * opt,
 		/*@null@*/ const char * translation_domain)
 	/*@globals fileSystem @*/
-	/*@modifies *fp, fileSystem @*/
+	/*@modifies fp, fileSystem @*/
 {
+    size_t maxLeftCol = columns->cur;
     size_t indentLength = maxLeftCol + 5;
-    size_t lineLength = _POPTHELP_MAXLINE - indentLength;
+    size_t lineLength = columns->max - indentLength;
     const char * help = D_(translation_domain, opt->descrip);
     const char * argDescrip = getArgDescrip(opt, translation_domain);
+    /* Display shortName iff printable non-space. */
+    int prtshort = (int)(isprint((int)opt->shortName) && opt->shortName != ' ');
     size_t helpLength;
     char * defs = NULL;
     char * left;
     size_t nb = maxLeftCol + 1;
     int displaypad = 0;
+    int xx;
 
     /* Make sure there's more than enough room in target buffer. */
     if (opt->longName)	nb += strlen(opt->longName);
     if (argDescrip)	nb += strlen(argDescrip);
 
-/*@-boundswrite@*/
     left = malloc(nb);
     if (left == NULL) return;	/* XXX can't happen */
     left[0] = '\0';
     left[maxLeftCol] = '\0';
 
-    if (opt->longName && opt->shortName)
-	sprintf(left, "-%c, %s%s", opt->shortName,
-		((opt->argInfo & POPT_ARGFLAG_ONEDASH) ? "-" : "--"),
-		opt->longName);
-    else if (opt->shortName != '\0') 
-	sprintf(left, "-%c", opt->shortName);
-    else if (opt->longName)
-	sprintf(left, "%s%s",
-		((opt->argInfo & POPT_ARGFLAG_ONEDASH) ? "-" : "--"),
-		opt->longName);
-    if (!*left) goto out;
+#define	prtlong	(opt->longName != NULL)	/* XXX splint needs a clue */
+    if (!(prtshort || prtlong))
+	goto out;
+    if (prtshort && prtlong) {
+	char *dash = F_ISSET(opt, ONEDASH) ? "-" : "--";
+	left[0] = '-';
+	left[1] = opt->shortName;
+	(void) stpcpy(stpcpy(stpcpy(left+2, ", "), dash), opt->longName);
+    } else if (prtshort) {
+	left[0] = '-';
+	left[1] = opt->shortName;
+	left[2] = '\0';
+    } else if (prtlong) {
+	/* XXX --long always padded for alignment with/without "-X, ". */
+	char *dash = poptArgType(opt) == POPT_ARG_MAINCALL ? ""
+		   : (F_ISSET(opt, ONEDASH) ? "-" : "--");
+	(void) stpcpy(stpcpy(stpcpy(left, "    "), dash), opt->longName);
+    }
+#undef	prtlong
 
     if (argDescrip) {
 	char * le = left + strlen(left);
 
-	if (opt->argInfo & POPT_ARGFLAG_OPTIONAL)
+	if (F_ISSET(opt, OPTIONAL))
 	    *le++ = '[';
 
 	/* Choose type of output */
-/*@-branchstate@*/
-	if (opt->argInfo & POPT_ARGFLAG_SHOW_DEFAULT) {
+	if (F_ISSET(opt, SHOW_DEFAULT)) {
 	    defs = singleOptionDefaultValue(lineLength, opt, translation_domain);
 	    if (defs) {
 		char * t = malloc((help ? strlen(help) : 0) +
 				strlen(defs) + sizeof(" "));
 		if (t) {
 		    char * te = t;
-		    *te = '\0';
-		    if (help) {
-			strcpy(te, help);	te += strlen(te);
-		    }
+		    if (help)
+			te = stpcpy(te, help);
 		    *te++ = ' ';
 		    strcpy(te, defs);
 		    defs = _free(defs);
@@ -283,17 +372,16 @@ static void singleOptionHelp(FILE * fp, size_t maxLeftCol,
 		defs = t;
 	    }
 	}
-/*@=branchstate@*/
 
 	if (opt->argDescrip == NULL) {
-	    switch (opt->argInfo & POPT_ARG_MASK) {
+	    switch (poptArgType(opt)) {
 	    case POPT_ARG_NONE:
 		break;
 	    case POPT_ARG_VAL:
 #ifdef	NOTNOW	/* XXX pug ugly nerdy output */
 	    {	long aLong = opt->val;
-		int ops = (opt->argInfo & POPT_ARGFLAG_LOGICALOPS);
-		int negate = (opt->argInfo & POPT_ARGFLAG_NOT);
+		int ops = F_ISSET(opt, LOGICALOPS);
+		int negate = F_ISSET(opt, NOT);
 
 		/* Don't bother displaying typical values */
 		if (!ops && (aLong == 0L || aLong == 1L || aLong == -1L))
@@ -323,77 +411,77 @@ static void singleOptionHelp(FILE * fp, size_t maxLeftCol,
 		break;
 	    case POPT_ARG_INT:
 	    case POPT_ARG_LONG:
+	    case POPT_ARG_LONGLONG:
 	    case POPT_ARG_FLOAT:
 	    case POPT_ARG_DOUBLE:
 	    case POPT_ARG_STRING:
 		*le++ = (opt->longName != NULL ? '=' : ' ');
-		strcpy(le, argDescrip);		le += strlen(le);
+		le = stpcpy(le, argDescrip);
 		break;
 	    default:
 		break;
 	    }
 	} else {
-	    size_t lelen;
+	    char *leo;
 
-	    *le++ = '=';
-	    strcpy(le, argDescrip);
-	    lelen = strlen(le);
-	    le += lelen;
+	    /* XXX argDescrip[0] determines "--foo=bar" or "--foo bar". */
+	    if (!strchr(" =(", argDescrip[0]))
+		*le++ = ((poptArgType(opt) == POPT_ARG_MAINCALL) ? ' ' :
+			 (poptArgType(opt) == POPT_ARG_ARGV) ? ' ' : '=');
+	    le = stpcpy(leo = le, argDescrip);
 
-#ifdef	POPT_WCHAR_HACK
-	    {	const char * scopy = argDescrip;
-		mbstate_t t;
-		size_t n;
-
-		memset ((void *)&t, 0, sizeof (t));	/* In initial state.  */
-		/* Determine number of characters.  */
-		n = mbsrtowcs (NULL, &scopy, strlen(scopy), &t);
-
-		displaypad = (int) (lelen-n);
-	    }
-#endif
+	    /* Adjust for (possible) wide characters. */
+	    displaypad = (int)((le - leo) - stringDisplayWidth(argDescrip));
 	}
-	if (opt->argInfo & POPT_ARGFLAG_OPTIONAL)
+	if (F_ISSET(opt, OPTIONAL))
 	    *le++ = ']';
 	*le = '\0';
     }
-/*@=boundswrite@*/
 
     if (help)
-	fprintf(fp,"  %-*s   ", (int)(maxLeftCol+displaypad), left);
+	xx = POPT_fprintf(fp,"  %-*s   ", (int)(maxLeftCol+displaypad), left);
     else {
-	fprintf(fp,"  %s\n", left); 
+	xx = POPT_fprintf(fp,"  %s\n", left); 
 	goto out;
     }
 
     left = _free(left);
-/*@-branchstate@*/
-    if (defs) {
+    if (defs)
 	help = defs;
-    }
 
     helpLength = strlen(help);
-/*@-boundsread@*/
     while (helpLength > lineLength) {
 	const char * ch;
 	char format[16];
 
 	ch = help + lineLength - 1;
-	while (ch > help && !isspace(*ch)) ch--;
+	while (ch > help && !_isspaceptr(ch))
+	    ch = POPT_prev_char(ch);
 	if (ch == help) break;		/* give up */
-	while (ch > (help + 1) && isspace(*ch)) ch--;
-	ch++;
+	while (ch > (help + 1) && _isspaceptr(ch))
+	    ch = POPT_prev_char (ch);
+	ch = POPT_next_char(ch);
+
+	/*
+	 *  XXX strdup is necessary to add NUL terminator so that an unknown
+	 *  no. of (possible) multi-byte characters can be displayed.
+	 */
+	{   char * fmthelp = xstrdup(help);
+	    if (fmthelp) {
+		fmthelp[ch - help] = '\0';
+		sprintf(format, "%%s\n%%%ds", (int) indentLength);
+		/*@-formatconst@*/
+		xx = POPT_fprintf(fp, format, fmthelp, " ");
+		/*@=formatconst@*/
+		free(fmthelp);
+	    }
+	}
 
-	sprintf(format, "%%.%ds\n%%%ds", (int) (ch - help), (int) indentLength);
-	/*@-formatconst@*/
-	fprintf(fp, format, help, " ");
-	/*@=formatconst@*/
 	help = ch;
-	while (isspace(*help) && *help) help++;
+	while (_isspaceptr(help) && *help)
+	    help = POPT_next_char(help);
 	helpLength = strlen(help);
     }
-/*@=boundsread@*/
-/*@=branchstate@*/
 
     if (helpLength) fprintf(fp, "%s\n", help);
     help = NULL;
@@ -417,49 +505,37 @@ static size_t maxArgWidth(const struct poptOption * opt,
 {
     size_t max = 0;
     size_t len = 0;
-    const char * s;
+    const char * argDescrip;
     
     if (opt != NULL)
     while (opt->longName || opt->shortName || opt->arg) {
-	if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_INCLUDE_TABLE) {
+	if (poptArgType(opt) == POPT_ARG_INCLUDE_TABLE) {
 	    if (opt->arg)	/* XXX program error */
 	        len = maxArgWidth(opt->arg, translation_domain);
 	    if (len > max) max = len;
-	} else if (!(opt->argInfo & POPT_ARGFLAG_DOC_HIDDEN)) {
+	} else if (!F_ISSET(opt, DOC_HIDDEN)) {
 	    len = sizeof("  ")-1;
-	    if (opt->shortName != '\0') len += sizeof("-X")-1;
-	    if (opt->shortName != '\0' && opt->longName) len += sizeof(", ")-1;
+	    /* XXX --long always padded for alignment with/without "-X, ". */
+	    len += sizeof("-X, ")-1;
 	    if (opt->longName) {
-		len += ((opt->argInfo & POPT_ARGFLAG_ONEDASH)
-			? sizeof("-")-1 : sizeof("--")-1);
+		len += (F_ISSET(opt, ONEDASH) ? sizeof("-") : sizeof("--")) - 1;
 		len += strlen(opt->longName);
 	    }
 
-	    s = getArgDescrip(opt, translation_domain);
+	    argDescrip = getArgDescrip(opt, translation_domain);
 
-#ifdef POPT_WCHAR_HACK
-	    /* XXX Calculate no. of display characters. */
-	    if (s) {
-		const char * scopy = s;
-		mbstate_t t;
-		size_t n;
+	    if (argDescrip) {
 
-/*@-boundswrite@*/
-		memset ((void *)&t, 0, sizeof (t));	/* In initial state.  */
-/*@=boundswrite@*/
-		/* Determine number of characters.  */
-		n = mbsrtowcs (NULL, &scopy, strlen(scopy), &t);
-		len += sizeof("=")-1 + n;
+		/* XXX argDescrip[0] determines "--foo=bar" or "--foo bar". */
+		if (!strchr(" =(", argDescrip[0])) len += sizeof("=")-1;
+
+		/* Adjust for (possible) wide characters. */
+		len += stringDisplayWidth(argDescrip);
 	    }
-#else
-	    if (s)
-		len += sizeof("=")-1 + strlen(s);
-#endif
 
-	    if (opt->argInfo & POPT_ARGFLAG_OPTIONAL) len += sizeof("[]")-1;
+	    if (F_ISSET(opt, OPTIONAL)) len += sizeof("[]")-1;
 	    if (len > max) max = len;
 	}
-
 	opt++;
     }
     
@@ -475,10 +551,11 @@ static size_t maxArgWidth(const struct poptOption * opt,
  * @param translation_domain	translation domain
  */
 static void itemHelp(FILE * fp,
-		/*@null@*/ poptItem items, int nitems, size_t left,
+		/*@null@*/ poptItem items, int nitems,
+		columns_t columns,
 		/*@null@*/ const char * translation_domain)
 	/*@globals fileSystem @*/
-	/*@modifies *fp, fileSystem @*/
+	/*@modifies fp, fileSystem @*/
 {
     poptItem item;
     int i;
@@ -487,9 +564,8 @@ static void itemHelp(FILE * fp,
     for (i = 0, item = items; i < nitems; i++, item++) {
 	const struct poptOption * opt;
 	opt = &item->option;
-	if ((opt->longName || opt->shortName) && 
-	    !(opt->argInfo & POPT_ARGFLAG_DOC_HIDDEN))
-	    singleOptionHelp(fp, left, opt, translation_domain);
+	if ((opt->longName || opt->shortName) && !F_ISSET(opt, DOC_HIDDEN))
+	    singleOptionHelp(fp, columns, opt, translation_domain);
     }
 }
 
@@ -498,43 +574,47 @@ static void itemHelp(FILE * fp,
  * @param con		context
  * @param fp		output file handle
  * @param table		option(s)
- * @param left		largest argument display width
+ * @param columns	output display width control
  * @param translation_domain	translation domain
  */
 static void singleTableHelp(poptContext con, FILE * fp,
-		/*@null@*/ const struct poptOption * table, size_t left,
+		/*@null@*/ const struct poptOption * table,
+		columns_t columns,
 		/*@null@*/ const char * translation_domain)
 	/*@globals fileSystem @*/
-	/*@modifies *fp, fileSystem @*/
+	/*@modifies fp, columns->cur, fileSystem @*/
 {
     const struct poptOption * opt;
     const char *sub_transdom;
+    int xx;
 
     if (table == poptAliasOptions) {
-	itemHelp(fp, con->aliases, con->numAliases, left, NULL);
-	itemHelp(fp, con->execs, con->numExecs, left, NULL);
+	itemHelp(fp, con->aliases, con->numAliases, columns, NULL);
+	itemHelp(fp, con->execs, con->numExecs, columns, NULL);
 	return;
     }
 
     if (table != NULL)
-    for (opt = table; (opt->longName || opt->shortName || opt->arg); opt++) {
-	if ((opt->longName || opt->shortName) && 
-	    !(opt->argInfo & POPT_ARGFLAG_DOC_HIDDEN))
-	    singleOptionHelp(fp, left, opt, translation_domain);
+    for (opt = table; opt->longName || opt->shortName || opt->arg; opt++) {
+	if ((opt->longName || opt->shortName) && !F_ISSET(opt, DOC_HIDDEN))
+	    singleOptionHelp(fp, columns, opt, translation_domain);
     }
 
     if (table != NULL)
-    for (opt = table; (opt->longName || opt->shortName || opt->arg); opt++) {
-	if ((opt->argInfo & POPT_ARG_MASK) != POPT_ARG_INCLUDE_TABLE)
+    for (opt = table; opt->longName || opt->shortName || opt->arg; opt++) {
+	if (poptArgType(opt) != POPT_ARG_INCLUDE_TABLE)
 	    continue;
 	sub_transdom = getTableTranslationDomain(opt->arg);
 	if (sub_transdom == NULL)
 	    sub_transdom = translation_domain;
 	    
+	/* If no popt aliases/execs, skip poptAliasOption processing. */
+	if (opt->arg == poptAliasOptions && !(con->numAliases || con->numExecs))
+	    continue;
 	if (opt->descrip)
-	    POPT_fprintf(fp, "\n%s\n", D_(sub_transdom, opt->descrip));
+	    xx = POPT_fprintf(fp, "\n%s\n", D_(sub_transdom, opt->descrip));
 
-	singleTableHelp(con, fp, opt->arg, left, sub_transdom);
+	singleTableHelp(con, fp, opt->arg, columns, sub_transdom);
     }
 }
 
@@ -544,20 +624,20 @@ static void singleTableHelp(poptContext con, FILE * fp,
  */
 static size_t showHelpIntro(poptContext con, FILE * fp)
 	/*@globals fileSystem @*/
-	/*@modifies *fp, fileSystem @*/
+	/*@modifies fp, fileSystem @*/
 {
     size_t len = (size_t)6;
     const char * fn;
+    int xx;
 
-    fprintf(fp, POPT_("Usage:"));
+    xx = POPT_fprintf(fp, POPT_("Usage:"));
     if (!(con->flags & POPT_CONTEXT_KEEP_FIRST)) {
-/*@-boundsread@*/
 /*@-type@*/	/* LCL: wazzup? */
 	fn = con->optionStack->argv[0];
 /*@=type@*/
-/*@=boundsread@*/
 	if (fn == NULL) return len;
 	if (strchr(fn, '/')) fn = strrchr(fn, '/') + 1;
+	/* XXX POPT_fprintf not needed for argv[0] display. */
 	fprintf(fp, " %s", fn);
 	len += strlen(fn) + 1;
     }
@@ -565,130 +645,119 @@ static size_t showHelpIntro(poptContext con, FILE * fp)
     return len;
 }
 
-void poptPrintHelp(poptContext con, FILE * fp, /*@unused@*/ int flags)
+void poptPrintHelp(poptContext con, FILE * fp, /*@unused@*/ UNUSED(int flags))
 {
-    size_t leftColWidth;
+    columns_t columns = calloc((size_t)1, sizeof(*columns));
+    int xx;
 
     (void) showHelpIntro(con, fp);
     if (con->otherHelp)
-	fprintf(fp, " %s\n", con->otherHelp);
+	xx = POPT_fprintf(fp, " %s\n", con->otherHelp);
     else
-	fprintf(fp, " %s\n", POPT_("[OPTION...]"));
+	xx = POPT_fprintf(fp, " %s\n", POPT_("[OPTION...]"));
 
-    leftColWidth = maxArgWidth(con->options, NULL);
-    singleTableHelp(con, fp, con->options, leftColWidth, NULL);
+    if (columns) {
+	columns->cur = maxArgWidth(con->options, NULL);
+	columns->max = maxColumnWidth(fp);
+	singleTableHelp(con, fp, con->options, columns, NULL);
+	free(columns);
+    }
 }
 
 /**
  * Display usage text for an option.
  * @param fp		output file handle
- * @param cursor	current display position
+ * @param columns	output display width control
  * @param opt		option(s)
  * @param translation_domain	translation domain
  */
-static size_t singleOptionUsage(FILE * fp, size_t cursor, 
+static size_t singleOptionUsage(FILE * fp, columns_t columns,
 		const struct poptOption * opt,
 		/*@null@*/ const char *translation_domain)
 	/*@globals fileSystem @*/
-	/*@modifies *fp, fileSystem @*/
+	/*@modifies fp, columns->cur, fileSystem @*/
 {
-    size_t len = (size_t)4;
-    char shortStr[2] = { '\0', '\0' };
-    const char * item = shortStr;
+    size_t len = sizeof(" []")-1;
     const char * argDescrip = getArgDescrip(opt, translation_domain);
-    int bingo = 0;
+    /* Display shortName iff printable non-space. */
+    int prtshort = (int)(isprint((int)opt->shortName) && opt->shortName != ' ');
 
-    if (opt->shortName != '\0' && opt->longName != NULL) {
-	len += 2;
-	if (!(opt->argInfo & POPT_ARGFLAG_ONEDASH)) len++;
+#define	prtlong	(opt->longName != NULL)	/* XXX splint needs a clue */
+    if (!(prtshort || prtlong))
+	return columns->cur;
+
+    len = sizeof(" []")-1;
+    if (prtshort)
+	len += sizeof("-c")-1;
+    if (prtlong) {
+	if (prtshort) len += sizeof("|")-1;
+	len += (F_ISSET(opt, ONEDASH) ? sizeof("-") : sizeof("--")) - 1;
 	len += strlen(opt->longName);
-	bingo++;
-    } else if (opt->shortName != '\0') {
-	len++;
-	shortStr[0] = opt->shortName;
-	shortStr[1] = '\0';
-	bingo++;
-    } else if (opt->longName) {
-	len += strlen(opt->longName);
-	if (!(opt->argInfo & POPT_ARGFLAG_ONEDASH)) len++;
-	item = opt->longName;
-	bingo++;
     }
 
-    if (!bingo) return cursor;
-
-#ifdef POPT_WCHAR_HACK
-    /* XXX Calculate no. of display characters. */
     if (argDescrip) {
-	const char * scopy = argDescrip;
-	mbstate_t t;
-	size_t n;
 
-/*@-boundswrite@*/
-	memset ((void *)&t, 0, sizeof (t));	/* In initial state.  */
-/*@=boundswrite@*/
-	/* Determine number of characters.  */
-	n = mbsrtowcs (NULL, &scopy, strlen(scopy), &t);
-	len += sizeof("=")-1 + n;
+	/* XXX argDescrip[0] determines "--foo=bar" or "--foo bar". */
+	if (!strchr(" =(", argDescrip[0])) len += sizeof("=")-1;
+
+	/* Adjust for (possible) wide characters. */
+	len += stringDisplayWidth(argDescrip);
     }
-#else
-    if (argDescrip) 
-	len += sizeof("=")-1 + strlen(argDescrip);
-#endif
 
-    if ((cursor + len) > _POPTHELP_MAXLINE) {
+    if ((columns->cur + len) > columns->max) {
 	fprintf(fp, "\n       ");
-	cursor = (size_t)7;
+	columns->cur = (size_t)7;
     } 
 
-    if (opt->longName && opt->shortName) {
-	fprintf(fp, " [-%c|-%s%s%s%s]",
-	    opt->shortName, ((opt->argInfo & POPT_ARGFLAG_ONEDASH) ? "" : "-"),
-	    opt->longName,
-	    (argDescrip ? " " : ""),
-	    (argDescrip ? argDescrip : ""));
-    } else {
-	fprintf(fp, " [-%s%s%s%s]",
-	    ((opt->shortName || (opt->argInfo & POPT_ARGFLAG_ONEDASH)) ? "" : "-"),
-	    item,
-	    (argDescrip ? (opt->shortName != '\0' ? " " : "=") : ""),
-	    (argDescrip ? argDescrip : ""));
-    }
+    fprintf(fp, " [");
+    if (prtshort)
+	fprintf(fp, "-%c", opt->shortName);
+    if (prtlong)
+	fprintf(fp, "%s%s%s",
+		(prtshort ? "|" : ""),
+		(F_ISSET(opt, ONEDASH) ? "-" : "--"),
+		opt->longName);
+#undef	prtlong
 
-    return cursor + len + 1;
+    if (argDescrip) {
+	/* XXX argDescrip[0] determines "--foo=bar" or "--foo bar". */
+	if (!strchr(" =(", argDescrip[0])) fprintf(fp, "=");
+	fprintf(fp, "%s", argDescrip);
+    }
+    fprintf(fp, "]");
+
+    return columns->cur + len + 1;
 }
 
 /**
  * Display popt alias and exec usage.
  * @param fp		output file handle
- * @param cursor	current display position
+ * @param columns	output display width control
  * @param item		alias/exec array
  * @param nitems	no. of ara/exec entries
  * @param translation_domain	translation domain
  */
-static size_t itemUsage(FILE * fp, size_t cursor,
+static size_t itemUsage(FILE * fp, columns_t columns,
 		/*@null@*/ poptItem item, int nitems,
 		/*@null@*/ const char * translation_domain)
 	/*@globals fileSystem @*/
-	/*@modifies *fp, fileSystem @*/
+	/*@modifies fp, columns->cur, fileSystem @*/
 {
     int i;
 
-/*@-branchstate@*/		/* FIX: W2DO? */
     if (item != NULL)
     for (i = 0; i < nitems; i++, item++) {
 	const struct poptOption * opt;
 	opt = &item->option;
-        if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_INTL_DOMAIN) {
+        if (poptArgType(opt) == POPT_ARG_INTL_DOMAIN) {
 	    translation_domain = (const char *)opt->arg;
-	} else if ((opt->longName || opt->shortName) &&
-		 !(opt->argInfo & POPT_ARGFLAG_DOC_HIDDEN)) {
-	    cursor = singleOptionUsage(fp, cursor, opt, translation_domain);
+	} else
+	if ((opt->longName || opt->shortName) && !F_ISSET(opt, DOC_HIDDEN)) {
+	    columns->cur = singleOptionUsage(fp, columns, opt, translation_domain);
 	}
     }
-/*@=branchstate@*/
 
-    return cursor;
+    return columns->cur;
 }
 
 /**
@@ -697,6 +766,7 @@ static size_t itemUsage(FILE * fp, size_t cursor,
 typedef struct poptDone_s {
     int nopts;
     int maxopts;
+/*@null@*/
     const void ** opts;
 } * poptDone;
 
@@ -704,31 +774,30 @@ typedef struct poptDone_s {
  * Display usage text for a table of options.
  * @param con		context
  * @param fp		output file handle
- * @param cursor	current display position
+ * @param columns	output display width control
  * @param opt		option(s)
  * @param translation_domain	translation domain
  * @param done		tables already processed
  * @return
  */
-static size_t singleTableUsage(poptContext con, FILE * fp, size_t cursor,
+static size_t singleTableUsage(poptContext con, FILE * fp, columns_t columns,
 		/*@null@*/ const struct poptOption * opt,
 		/*@null@*/ const char * translation_domain,
 		/*@null@*/ poptDone done)
 	/*@globals fileSystem @*/
-	/*@modifies *fp, done, fileSystem @*/
+	/*@modifies fp, columns->cur, done, fileSystem @*/
 {
-/*@-branchstate@*/		/* FIX: W2DO? */
     if (opt != NULL)
     for (; (opt->longName || opt->shortName || opt->arg) ; opt++) {
-        if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_INTL_DOMAIN) {
+        if (poptArgType(opt) == POPT_ARG_INTL_DOMAIN) {
 	    translation_domain = (const char *)opt->arg;
-	} else if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_INCLUDE_TABLE) {
+	} else
+	if (poptArgType(opt) == POPT_ARG_INCLUDE_TABLE) {
 	    if (done) {
 		int i = 0;
+		if (done->opts != NULL)
 		for (i = 0; i < done->nopts; i++) {
-/*@-boundsread@*/
 		    const void * that = done->opts[i];
-/*@=boundsread@*/
 		    if (that == NULL || that != opt->arg)
 			/*@innercontinue@*/ continue;
 		    /*@innerbreak@*/ break;
@@ -736,21 +805,18 @@ static size_t singleTableUsage(poptContext con, FILE * fp, size_t cursor,
 		/* Skip if this table has already been processed. */
 		if (opt->arg == NULL || i < done->nopts)
 		    continue;
-/*@-boundswrite@*/
-		if (done->nopts < done->maxopts)
+		if (done->opts != NULL && done->nopts < done->maxopts)
 		    done->opts[done->nopts++] = (const void *) opt->arg;
-/*@=boundswrite@*/
 	    }
-	    cursor = singleTableUsage(con, fp, cursor, opt->arg,
+	    columns->cur = singleTableUsage(con, fp, columns, opt->arg,
 			translation_domain, done);
-	} else if ((opt->longName || opt->shortName) &&
-		 !(opt->argInfo & POPT_ARGFLAG_DOC_HIDDEN)) {
-	    cursor = singleOptionUsage(fp, cursor, opt, translation_domain);
+	} else
+	if ((opt->longName || opt->shortName) && !F_ISSET(opt, DOC_HIDDEN)) {
+	    columns->cur = singleOptionUsage(fp, columns, opt, translation_domain);
 	}
     }
-/*@=branchstate@*/
 
-    return cursor;
+    return columns->cur;
 }
 
 /**
@@ -764,69 +830,77 @@ static size_t singleTableUsage(poptContext con, FILE * fp, size_t cursor,
 static size_t showShortOptions(const struct poptOption * opt, FILE * fp,
 		/*@null@*/ char * str)
 	/*@globals fileSystem @*/
-	/*@modifies *str, *fp, fileSystem @*/
+	/*@modifies str, *fp, fileSystem @*/
 	/*@requires maxRead(str) >= 0 @*/
 {
     /* bufsize larger then the ascii set, lazy allocation on top level call. */
     size_t nb = (size_t)300;
-    char * s = (str != NULL ? str : calloc(1, nb));
+    char * s = (str != NULL ? str : calloc((size_t)1, nb));
     size_t len = (size_t)0;
 
     if (s == NULL)
 	return 0;
 
-/*@-boundswrite@*/
     if (opt != NULL)
     for (; (opt->longName || opt->shortName || opt->arg); opt++) {
-	if (opt->shortName && !(opt->argInfo & POPT_ARG_MASK))
-	    s[strlen(s)] = opt->shortName;
-	else if ((opt->argInfo & POPT_ARG_MASK) == POPT_ARG_INCLUDE_TABLE)
+	if (!F_ISSET(opt, DOC_HIDDEN) && opt->shortName && !poptArgType(opt))
+	{
+	    /* Display shortName iff unique printable non-space. */
+	    if (!strchr(s, opt->shortName) && isprint((int)opt->shortName)
+	     && opt->shortName != ' ')
+		s[strlen(s)] = opt->shortName;
+	} else if (poptArgType(opt) == POPT_ARG_INCLUDE_TABLE)
 	    if (opt->arg)	/* XXX program error */
 		len = showShortOptions(opt->arg, fp, s);
     } 
-/*@=boundswrite@*/
 
     /* On return to top level, print the short options, return print length. */
     if (s != str && *s != '\0') {
 	fprintf(fp, " [-%s]", s);
 	len = strlen(s) + sizeof(" [-]")-1;
     }
+/*@-temptrans@*/	/* LCL: local s, not str arg, is being freed. */
     if (s != str)
 	free(s);
+/*@=temptrans@*/
     return len;
 }
 
-void poptPrintUsage(poptContext con, FILE * fp, /*@unused@*/ int flags)
+void poptPrintUsage(poptContext con, FILE * fp, /*@unused@*/ UNUSED(int flags))
 {
+    columns_t columns = calloc((size_t)1, sizeof(*columns));
     struct poptDone_s done_buf;
     poptDone done = &done_buf;
-    size_t cursor;
 
     memset(done, 0, sizeof(*done));
     done->nopts = 0;
     done->maxopts = 64;
-    cursor = done->maxopts * sizeof(*done->opts);
-/*@-boundswrite@*/
-    done->opts = calloc(1, cursor);
+  if (columns) {
+    columns->cur = done->maxopts * sizeof(*done->opts);
+    columns->max = maxColumnWidth(fp);
+    done->opts = calloc((size_t)1, columns->cur);
     /*@-keeptrans@*/
-    done->opts[done->nopts++] = (const void *) con->options;
+    if (done->opts != NULL)
+	done->opts[done->nopts++] = (const void *) con->options;
     /*@=keeptrans@*/
-/*@=boundswrite@*/
 
-    cursor = showHelpIntro(con, fp);
-    cursor += showShortOptions(con->options, fp, NULL);
-    cursor = singleTableUsage(con, fp, cursor, con->options, NULL, done);
-    cursor = itemUsage(fp, cursor, con->aliases, con->numAliases, NULL);
-    cursor = itemUsage(fp, cursor, con->execs, con->numExecs, NULL);
+    columns->cur = showHelpIntro(con, fp);
+    columns->cur += showShortOptions(con->options, fp, NULL);
+    columns->cur = singleTableUsage(con, fp, columns, con->options, NULL, done);
+    columns->cur = itemUsage(fp, columns, con->aliases, con->numAliases, NULL);
+    columns->cur = itemUsage(fp, columns, con->execs, con->numExecs, NULL);
 
     if (con->otherHelp) {
-	cursor += strlen(con->otherHelp) + 1;
-	if (cursor > _POPTHELP_MAXLINE) fprintf(fp, "\n       ");
+	columns->cur += strlen(con->otherHelp) + 1;
+	if (columns->cur > columns->max) fprintf(fp, "\n       ");
 	fprintf(fp, " %s", con->otherHelp);
     }
 
     fprintf(fp, "\n");
-    free(done->opts);
+    if (done->opts != NULL)
+	free(done->opts);
+    free(columns);
+  }
 }
 
 void poptSetOtherOptionHelp(poptContext con, const char * text)
