@@ -6,8 +6,8 @@
 
 /*@unchecked@*/
 static int pass2 = 0;
-static void option_callback(/*@unused@*/ poptContext con,
-		/*@unused@*/ enum poptCallbackReason reason,
+static void option_callback(/*@unused@*/ UNUSED(poptContext con),
+		/*@unused@*/ UNUSED(enum poptCallbackReason reason),
 		const struct poptOption * opt,
 		char * arg, void * data)
 	/*@globals fileSystem @*/
@@ -46,6 +46,10 @@ static long aLong = 738905609L;
 /*@unchecked@*/
 static long bLong = 738905609L;
 /*@unchecked@*/
+static long long aLongLong = 738905609LL;
+/*@unchecked@*/
+static long long bLongLong = 738905609LL;
+/*@unchecked@*/
 static float aFloat = 3.1415926535;
 /*@unchecked@*/
 static float bFloat = 3.1415926535;
@@ -53,6 +57,9 @@ static float bFloat = 3.1415926535;
 static double aDouble = 9.86960440108935861883;
 /*@unchecked@*/
 static double bDouble = 9.86960440108935861883;
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+static const char ** aArgv = NULL;
+
 /*@unchecked@*/ /*@null@*/
 static char * oStr = (char *) -1;
 /*@unchecked@*/
@@ -121,11 +128,23 @@ static struct poptOption options[] = {
 	"POPT_ARG_INT: 271828", NULL },
   { "long", 'l', POPT_ARG_LONG | POPT_ARGFLAG_SHOW_DEFAULT, &aLong, 0,
 	"POPT_ARG_LONG: 738905609", NULL },
+  { "longlong", 'L', POPT_ARG_LONGLONG | POPT_ARGFLAG_SHOW_DEFAULT, &aLongLong, 0,
+	"POPT_ARG_LONGLONG: 738905609", NULL },
   { "float", 'f', POPT_ARG_FLOAT | POPT_ARGFLAG_SHOW_DEFAULT, &aFloat, 0,
 	"POPT_ARG_FLOAT: 3.14159", NULL },
   { "double", 'd', POPT_ARG_DOUBLE | POPT_ARGFLAG_SHOW_DEFAULT, &aDouble, 0,
 	"POPT_ARG_DOUBLE: 9.8696", NULL },
 
+   { "randint", '\0', POPT_ARG_INT|POPT_ARGFLAG_RANDOM, &aInt, 0,
+	"POPT_ARGFLAG_RANDOM: experimental", NULL },
+   { "randlong", '\0', POPT_ARG_LONG|POPT_ARGFLAG_RANDOM, &aLong, 0,
+	"POPT_ARGFLAG_RANDOM: experimental", NULL },
+   { "randlonglong", '\0', POPT_ARG_LONGLONG|POPT_ARGFLAG_RANDOM, &aLongLong, 0,
+	"POPT_ARGFLAG_RANDOM: experimental", NULL },
+
+   { "argv", '\0', POPT_ARG_ARGV, &aArgv, 0,
+	"POPT_ARG_ARGV: append arg to array (can be used multiple times)",NULL},
+
   { "bitset", '\0', POPT_BIT_SET | POPT_ARGFLAG_SHOW_DEFAULT, &aFlag, 0x4321,
 	"POPT_BIT_SET: |= 0x4321", 0},
   { "bitclr", '\0', POPT_BIT_CLR | POPT_ARGFLAG_SHOW_DEFAULT, &aFlag, 0x1234,
@@ -148,10 +167,10 @@ static struct poptOption options[] = {
 
 static void resetVars(void)
 	/*@globals arg1, arg2, arg3, inc, shortopt,
-		aVal, aFlag, aInt, aLong, aFloat, aDouble,
+		aVal, aFlag, aInt, aLong, aLongLong, aFloat, aDouble, aArgv,
 		oStr, singleDash, pass2 @*/
 	/*@modifies arg1, arg2, arg3, inc, shortopt,
-		aVal, aFlag, aInt, aLong, aFloat, aDouble,
+		aVal, aFlag, aInt, aLong, aLongLong, aFloat, aDouble, aArgv,
 		oStr, singleDash, pass2 @*/
 {
     arg1 = 0;
@@ -165,9 +184,22 @@ static void resetVars(void)
 
     aInt = bInt;
     aLong = bLong;
+    aLongLong = bLongLong;
     aFloat = bFloat;
     aDouble = bDouble;
 
+    if (aArgv) {
+	int i;
+	for (i = 0; aArgv[i] != NULL; i++) {
+/*@-unqualifiedtrans@*/
+	    free((void *)aArgv[i]);
+/*@=unqualifiedtrans@*/
+	    aArgv[i] = NULL;
+	}
+	free(aArgv);
+	aArgv = NULL;
+    }
+
     oStr = (char *) -1;
 
     singleDash = 0;
@@ -243,18 +275,26 @@ int main(int argc, const char ** argv)
 	fprintf(stdout, " aInt: %d", aInt);
     if (aLong != bLong)
 	fprintf(stdout, " aLong: %ld", aLong);
+    if (aLongLong != bLongLong)
+	fprintf(stdout, " aLongLong: %lld", aLongLong);
 /*@-realcompare@*/
     if (aFloat != bFloat)
 	fprintf(stdout, " aFloat: %g", (double)aFloat);
     if (aDouble != bDouble)
 	fprintf(stdout, " aDouble: %g", aDouble);
 /*@=realcompare@*/
+    if (aArgv != NULL) {
+	const char **av = aArgv;
+	const char * arg;
+	fprintf(stdout, " aArgv:");
+	while ((arg = *av++) != NULL)
+	    fprintf(stdout, " %s", arg);
+    }
     if (oStr != (char *)-1)
 	fprintf(stdout, " oStr: %s", (oStr ? oStr : "(none)"));
     if (singleDash)
 	fprintf(stdout, " -");
 
-/*@-boundsread@*/
     rest = poptGetArgs(optCon);
     if (rest) {
 	fprintf(stdout, " rest:");
@@ -263,7 +303,6 @@ int main(int argc, const char ** argv)
 	    rest++;
 	}
     }
-/*@=boundsread@*/
 
     fprintf(stdout, "\n");
 
